from viren import get_names
from viren import do_rename
from viren import VirenError

import nose
import os
import tempfile


def make_fs(fs):
    """
    Create a testing FS in a temp dir, return path to dir.

    FS is given as a list of (filename, contents) tuples.
    """
    dir_path = tempfile.mkdtemp(prefix='viren-test-')
    for filename, contents in fs:
        path = os.path.join(dir_path, filename)
        open(path, 'w').write(contents)
    return dir_path


def read_fs(dir_path):
    """
    Read files in dir_path.

    Return FS as list of (filename, contents) tuples, where the filenames are
    in sorted order.
    """
    fs = []
    for name in os.listdir(dir_path):
        path = os.path.join(dir_path, name)
        fs.append((name, open(path).read()))
    fs.sort()
    return fs


def remove_fs(dir_path):
    """
    Remove files in dir_path and then the directory itself.
    """
    for name in os.listdir(dir_path):
        os.remove(os.path.join(dir_path, name))
    os.rmdir(dir_path)


def check_success(old_fs, new_fs, old_names, new_names):
    """
    Create old_fs, rename old_names -> new_names, compare with new_fs.

    Expects the rename to succeed; raises an error otherwise.
    """
    dir_path = make_fs(old_fs)
    got_old_names = get_names(dir_path)
    nose.tools.assert_equal(got_old_names, old_names)
    do_rename(dir_path, new_names)
    got_new_fs = read_fs(dir_path)
    nose.tools.assert_equal(got_new_fs, new_fs)
    remove_fs(dir_path)


def check_failure(old_fs, new_fs, old_names, new_names, message):
    """
    Create old_fs, rename old_names -> new_names, compare with new_fs.

    Expects the rename to fail with an error containing message.
    """
    dir_path = make_fs(old_fs)
    got_old_names = get_names(dir_path)
    nose.tools.assert_equal(got_old_names, old_names)
    with nose.tools.assert_raises(VirenError) as cm:
        do_rename(dir_path, new_names)
    nose.tools.assert_in(message, cm.exception.message)
    got_new_fs = read_fs(dir_path)
    nose.tools.assert_equal(got_new_fs, new_fs)
    remove_fs(dir_path)


def test_simple():
    """
    Simple rename works.
    """
    old_fs = [('a', '1'), ('b', '2'), ('c', '3')]
    new_fs = [('aa', '1'), ('bb', '2'), ('cc', '3')]
    old_names = ['a', 'b', 'c']
    new_names = ['aa', 'bb', 'cc']
    check_success(old_fs, new_fs, old_names, new_names)


def test_loop():
    """
    Renaming files in a loop works.
    """
    old_fs = [('a', '1'), ('b', '2'), ('c', '3'), ('d', '4')]
    new_fs = [('a', '4'), ('b', '1'), ('c', '2'), ('d', '3')]
    old_names = ['a', 'b', 'c', 'd']
    new_names = ['b', 'c', 'd', 'a']
    check_success(old_fs, new_fs, old_names, new_names)


def test_spaces():
    """
    Spaces in file names are OK.
    """
    old_fs = [('a', '1'), ('a a', '2'), ('a a a', '3')]
    new_fs = [('b b', '1'), ('c', '2'), ('d d d', '3')]
    old_names = ['a', 'a a', 'a a a']
    new_names = ['b b', 'c', 'd d d']
    check_success(old_fs, new_fs, old_names, new_names)


def test_spaces_on_edge():
    """
    Spaces at the beginning or end of a filename are ignored.
    """
    old_fs = [('a', '1'), ('b', '2'), ('c', '3')]
    new_fs = [('aa', '1'), ('bb', '2'), ('cc', '3')]
    old_names = ['a', 'b', 'c']
    new_names = ['aa  ', '   bb', '  cc   ']
    check_success(old_fs, new_fs, old_names, new_names)


def test_slashes():
    """
    Slashes in filenames are not allowed.
    """
    old_fs = [('a', '1'), ('b', '2'), ('c', '3')]
    new_fs = [('a', '1'), ('b', '2'), ('c', '3')]
    old_names = ['a', 'b', 'c']
    new_names = ['a/a', 'bb', 'cc']
    check_failure(old_fs, new_fs, old_names, new_names, "slash")


def test_mismatch():
    """
    Fails if too many or too few names provided.
    """
    old_fs = [('a', '1'), ('b', '2'), ('c', '3')]
    new_fs = [('a', '1'), ('b', '2'), ('c', '3')]
    old_names = ['a', 'b', 'c']
    new_names = ['aa', 'bb', 'cc', 'dd']
    check_failure(old_fs, new_fs, old_names, new_names, "list has")

    old_fs = [('a', '1'), ('b', '2'), ('c', '3')]
    new_fs = [('a', '1'), ('b', '2'), ('c', '3')]
    old_names = ['a', 'b', 'c']
    new_names = ['aa', 'bb']
    check_failure(old_fs, new_fs, old_names, new_names, "list has")


def test_collision():
    old_fs = [('a', '1'), ('b', '2'), ('c', '3')]
    new_fs = [('a', '1'), ('b', '2'), ('c', '3')]
    old_names = ['a', 'b', 'c']
    new_names = ['aa', 'aa', 'cc']
    check_failure(old_fs, new_fs, old_names, new_names, "duplicate")


def test_dots():
    old_fs = [('a', '1'), ('b', '2'), ('c', '3')]
    new_fs = [('a', '1'), ('b', '2'), ('c', '3')]
    old_names = ['a', 'b', 'c']
    new_names = ['aa', 'bb', '.']
    check_failure(old_fs, new_fs, old_names, new_names, ". or ..")

    old_fs = [('a', '1'), ('b', '2'), ('c', '3')]
    new_fs = [('a', '1'), ('b', '2'), ('c', '3')]
    old_names = ['a', 'b', 'c']
    new_names = ['aa', '..', 'cc']
    check_failure(old_fs, new_fs, old_names, new_names, ". or ..")


# TODO:
# - unit tests for mk_temp_subdir
# - unit tests for verify_rename_list instead of only end-to-end tests



import distribute_setup
distribute_setup.use_setuptools()

from setuptools import setup

from viren import __version__


long_description = \
"""
See docs and report bugs at https://github.com/cberzan/viren.
"""

setup(
    name = "viren",
    version = __version__,
    packages = ["viren"],
    py_modules = ["distribute_setup"],

    # metadata for upload to PyPI
    author = "Constantin Berzan",
    author_email = "cberzan@gmail.com",
    description = "bulk-rename files using your editor",
    long_description = long_description,
    license = "MIT",
    keywords = "rename",
    url = "https://github.com/cberzan/viren",
    # download_url =

    classifiers = [
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 2.7",
    ],

    entry_points = {
        'console_scripts': [
            'viren = viren.viren:main',
        ],
    },
)



#!/usr/bin/env python

import os
import random
import subprocess
import sys
import tempfile


class VirenError(RuntimeError):
    pass


def verify_rename_list(old_names, new_names):
    """
    Sanity check a rename proposal.

    If there is a problem (new names not provided, or they collide among
    themselves, etc), raises VirenError.
    """
    if len(old_names) != len(new_names):
        raise VirenError("Edited list has {} names, expected {}".format(
            len(new_names), len(old_names)))
    seen_names = set()
    for lineno, name in enumerate(new_names, 1):
        if not name:
            raise VirenError("Line {}: empty filename".format(lineno))
        if '/' in name:
            raise VirenError("Line {}: slash in filename".format(lineno))
        if name in ('.', '..'):
            raise VirenError("Line {}: filename is . or ..".format(lineno))
        if name in seen_names:
            raise VirenError("Line {}: duplicate filename".format(lineno))
        seen_names.add(name)


def mk_temp_subdir(dir_path, forbidden_names):
    """
    Make a temp subdir in pwd and return its path.

    The directory name is assigned randomly, and checked not to collide with
    `forbidden_names`.
    """
    for attempt in xrange(10):
        name = 'viren-' + hex(random.randint(1e9, 1e10))[2:]
        path = os.path.join(dir_path, name)
        if name in forbidden_names or os.path.exists(path):
            continue
        os.mkdir(path)
        return path
    raise VirenError("Failed to create temp subdir")


def get_names(dir_path):
    """
    Return list of filenames in given dir, in sorted order.
    """
    names = os.listdir(dir_path)
    names.sort()
    return names


def do_rename(dir_path, new_names):
    """
    Rename files in dir_path (in sorted order) to new_names.

    Leading or trailing whitespace in `new_names` is ignored.

    Raises VirenError if the rename cannot be performed.
    """
    # Clean new_names and do some sanity checks.
    new_names = [name.strip() for name in new_names]
    old_names = get_names(dir_path)
    verify_rename_list(old_names, new_names)

    # Move the old files to a temp subdir. We do this to avoid accidental
    # overwrites, in case old_names and new_names overlap.
    subdir_path = mk_temp_subdir(dir_path, old_names + new_names)
    for name in old_names:
        old_path = os.path.join(dir_path, name)
        new_path = os.path.join(subdir_path, name)
        ret = subprocess.call(['mv', old_path, new_path])
        if ret != 0:
            raise VirenError('mv failed with return code {}'.format(ret))

    # Now move the files back with their new names.
    for i in xrange(len(old_names)):
        old_path = os.path.join(subdir_path, old_names[i])
        new_path = os.path.join(dir_path, new_names[i])
        ret = subprocess.call(['mv', old_path, new_path])
        if ret != 0:
            raise VirenError('mv failed with return code {}'.format(ret))

    # Clean up.
    os.rmdir(subdir_path)


def main():
    """
    Run viren in the current directory.
    """
    # Get sorted list of files in pwd.
    old_names = get_names('.')

    # Write that list to a temp file.
    _, temp_path = tempfile.mkstemp(prefix='viren-')
    temp = open(temp_path, 'w')
    temp.write('\n'.join(old_names))
    temp.close()

    try:
        # Let the user edit that file in her favorite editor.
        ret = subprocess.call(['editor', temp_path])
        if ret != 0:
            raise VirenError('editor failed with return code {}'.format(ret))

        # Get the edited list back from the temp file, and do the rename.
        new_names = open(temp_path).readlines()
        do_rename('.', new_names)

        # Clean up.
        os.remove(temp_path)
        print "Done."

    except VirenError as err:
        print >>sys.stderr, "Something went wrong:"
        print >>sys.stderr, err.message
        print >>sys.stderr, "File list saved to {}".format(temp_path)
        sys.exit(1)


if __name__ == "__main__":
    main()
